%
%
%  Time Subtract Average Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="TimeSubtractAverage" icon="Matlab" tooltip="For each voxel, subtract the temporal average intensity.">
%          <Command>MatlabXT::XTTimeSubtractAverage(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%   
%   For each voxel, subtract the temporal average intensity.
%   This function is used to eliminate motionless objects.
%   For 2D time series try to use 'Image processing - Swap Time and Z' 
%     before and after starting the XTension to speed up processing.
%
%

function XTTimeSubtractAverage(aImarisApplicationID)

% get the application object
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  % called from workspace
  vImarisApplication = aImarisApplicationID;
else
  % connect to Imaris interface
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% get the data set 
vDataSet = vImarisApplication.GetDataSet.Clone;

try
  vImarisApplication.DataSetPushUndo('TimeSubtractAverage');
catch er
  er.message;
  % nothing to do
end

try % dataset has been loaded?
  vSize = [vDataSet.GetSizeX, vDataSet.GetSizeY, vDataSet.GetSizeZ];
catch er
  msgbox(sprintf('Invalid dataset \n\n%s', er.message))
  return
end

vTotalCount = vDataSet.GetSizeC * vDataSet.GetSizeT * 3;

vIsTime = 1;
if vDataSet.GetSizeT ==1
  vAnswer = questdlg(['This is not a time dataset. Would you like to ', ...
    'subtract the average of the slices instead?'], 'TimeSubtractAverage');
  if ~isequal(vAnswer, 'Yes')
    return
  end
  vIsTime = 0;

  vAtOnce = inputdlg('Please enter the number of slices to process at once:', ...
    'TimeSubtractAverage', 1, {'100'});
  vAtOnce = str2double(vAtOnce{1});
  if isnan(vAtOnce)
    msgbox('Invalid entry')
    return
  end
  vTotalCount = vTotalCount * numel(0:vAtOnce:vSize(3)-1);
end

[vDiscardMode, vOk] = listdlg('ListString', {'Set to zero', 'Get absolute value'},...
                       'SelectionMode','single',...
                       'ListSize',[200 50],...
                       'Name','TimeSubtractAverage',...
                       'PromptString', {'Neagative values:'});
if vOk<1
    return;
end

vProgressDisplay = waitbar(0, 'TimeSubtractAverage');
vCount = 0;

for vChannel = 0:vDataSet.GetSizeC-1
  if vImarisApplication.GetChannelVisibility(vChannel)
    if vIsTime
      % compute mean volume
      vMeanVolume = double(zeros(prod(vSize), 1));
      for vTime = 0:vDataSet.GetSizeT-1
        vVolume = double(vDataSet.GetDataVolumeAs1DArrayFloats(vChannel, vTime));
        vMeanVolume = (vMeanVolume*vTime + vVolume) / (vTime+1);
        vCount = vCount + 1;
        waitbar(vCount/vTotalCount)
      end

      % subtract mean volume
      vMin = zeros(vDataSet.GetSizeT, 1) + inf;
      vMax = zeros(vDataSet.GetSizeT, 1) - inf;
      for vTime = 0:vDataSet.GetSizeT-1
        vVolume = double(vDataSet.GetDataVolumeAs1DArrayFloats(vChannel, vTime));
        vVolume = reshape(vVolume - vMeanVolume, vSize);

        if vDiscardMode
          vVolume(vVolume<0) = 0;
        else
          vVolume = abs(vVolume);
        end

        vMin(vTime+1) = min(vVolume(:));
        vMax(vTime+1) = max(vVolume(:));
        if strcmp(vDataSet.GetType,'eTypeUInt8')
          vDataSet.SetDataVolumeBytes(uint8(vVolume), vChannel, vTime);
        elseif strcmp(vDataSet.GetType,'eTypeUInt16')
          vDataSet.SetDataVolumeShorts(uint16(vVolume), vChannel, vTime);
        elseif strcmp(vDataSet.GetType,'eTypeFloat')
          vDataSet.SetDataVolumeFloats(single(vVolume), vChannel, vTime);
        end
        vCount = vCount + 2;
        waitbar(vCount/vTotalCount)
      end

    else % vIstime == false

      % compute mean volume
      vMeanVolume = double(zeros(vSize(1:2)));
      for vBlock = 0:vAtOnce:vSize(3)-1
        vMinZ = vBlock;
        vSizeZ = min(vBlock + vAtOnce, vSize(3)) - vBlock;
        vVolume = double(vDataSet.GetDataSubVolumeAs1DArrayFloats( ...
          0, 0, vMinZ, vChannel, 0, vSize(1), vSize(2), vSizeZ));

        vVolume = reshape(vVolume, vSize(1), vSize(2), []);

        vMeanVolume = (vMeanVolume*vMinZ + mean(vVolume, 3)*vSizeZ) / ...
          (vMinZ + vSizeZ);

        vCount = vCount + 1;
        waitbar(vCount/vTotalCount)
      end

      % subtract mean volume
      vMin = zeros(numel(0:vAtOnce:vSize(3)-1), 1) + inf;
      vMax = zeros(numel(0:vAtOnce:vSize(3)-1), 1) - inf;
      for vBlock = 0:vAtOnce:vSize(3)-1
        vMinZ = vBlock;
        vSizeZ = min(vBlock + vAtOnce, vSize(3)) - vBlock;
        vVolume = double(vDataSet.GetDataSubVolumeAs1DArrayFloats( ...
          0, 0, vMinZ, vChannel, 0, vSize(1), vSize(2), vSizeZ));

        vVolume = reshape(vVolume, [vSize(1:2), vSizeZ]);
        for vSlice = 1:vSizeZ
          vVolume(:, :, vSlice) = vVolume(:, :, vSlice) - vMeanVolume;
        end

        if vDiscardMode
          vVolume(vVolume<0) = 0;
        else
          vVolume = abs(vVolume);
        end

        vMin(vBlock+1) = min(vVolume(:));
        vMax(vBlock+1) = max(vVolume(:));
        if strcmp(vDataSet.GetType,'eTypeUInt8')
          vDataSet.SetDataSubVolumeBytes(uint8(vVolume), 0, 0, vMinZ, vChannel, 0);
        elseif strcmp(vDataSet.GetType,'eTypeUInt16')
          vDataSet.SetDataSubVolumeShorts(uint16(vVolume), 0, 0, vMinZ, vChannel, 0);
        elseif strcmp(vDataSet.GetType,'eTypeFloat')
          vDataSet.SetDataSubVolumeFloats(single(vVolume), 0, 0, vMinZ, vChannel, 0);
        end

        vCount = vCount + 2;
        waitbar(vCount/vTotalCount)
      end

    end
    vDataSet.SetChannelRange(vChannel, min(vMin), max(vMax));
  end
end
close(vProgressDisplay)

vImarisApplication.SetDataSet(vDataSet);
